@@ -12,6 +12,9 @@ from sbi.inference.snpe.snpe_c import SNPE_C  # noqa: F401
 
 from sbi.inference.snre import SNRE, SNRE_A, SNRE_B  # noqa: F401
 
+from sbi.inference.abc.mcabc import MCABC
+from sbi.inference.abc.smcabc import SMCABC
+
 
 SNL = SNLE = SNLE_A
 _snle_family = ["SNL"]
@@ -25,5 +28,9 @@ SRE = SNRE_B
 AALR = SNRE_A
 _snre_family = ["SNRE_A", "AALR", "SNRE_B", "SNRE", "SRE"]
 
+ABC = MCABC
+SMC = SMCABC
+_abc_family = ["ABC", "MCABC", "SMC", "SMCABC"]
 
-__all__ = _snpe_family + _snre_family + _snle_family
+
+__all__ = _snpe_family + _snre_family + _snle_family + _abc_family
