@@ -1,15 +1,14 @@
 from __future__ import annotations
 
-from typing import Optional, Callable, Union
+from typing import Callable, Optional, Union
 
 import torch
-from torch import nn, ones, Tensor
+from torch import Tensor
 from torch.utils.tensorboard import SummaryWriter
 
 from sbi.inference.posterior import NeuralPosterior
-from sbi.types import OneOrMore
 from sbi.inference.snre.snre_base import RatioEstimator
-from sbi.utils.torchutils import get_default_device
+from sbi.types import OneOrMore
 from sbi.utils import del_entries
 
 
@@ -18,13 +17,11 @@ class SNRE_B(RatioEstimator):
         self,
         simulator: Callable,
         prior,
-        x_shape: Optional[torch.Size] = None,
         num_workers: int = 1,
         simulation_batch_size: int = 1,
-        embedding_net: nn.Module = nn.Identity(),
-        classifier: Union[str, nn.Module] = "resnet",
+        classifier: Union[str, Callable] = "resnet",
         mcmc_method: str = "slice_np",
-        device: Union[torch.device, str] = get_default_device(),
+        device: str = "cpu",
         logging_level: Union[int, str] = "warning",
         summary_writer: Optional[SummaryWriter] = None,
         show_progress_bars: bool = True,
@@ -44,19 +41,21 @@ class SNRE_B(RatioEstimator):
                 parameters, e.g. which ranges are meaningful for them. Any
                 object with `.log_prob()`and `.sample()` (for example, a PyTorch
                 distribution) can be used.
-            x_shape: Shape of a single simulation output $x$, has to be (1,N).
             num_workers: Number of parallel workers to use for simulations.
             simulation_batch_size: Number of parameter sets that the simulator
                 maps to data x at once. If None, we simulate all parameter sets at the
                 same time. If >= 1, the simulator has to process data of shape
                 (simulation_batch_size, parameter_dimension).
-            embedding_net: Can be used to encode observations $x$. Currently not
-                implemented.
-            classifier: Classifier trained to approximate likelihood rations. If str,
-                use a pre-configured neural network.
-            mcmc_method: If MCMC sampling is used, specify the method here: either of
+            classifier: Classifier trained to approximate likelihood ratios. If it is
+                a string, use a pre-configured network of the provided type (one of
+                linear, mlp, resnet). Alternatively, a function that builds a custom
+                neural network can be provided. The function will be called with the
+                first batch of simulations (theta, x), which can thus be used for shape
+                inference and potentially for z-scoring. It needs to return a PyTorch
+                `nn.Module` implementing the classifier.
+            mcmc_method: Specify the method for MCMC sampling, either of:
                 slice_np, slice, hmc, nuts.
-            device: torch device on which to compute, e.g. cuda, cpu.
+            device: torch device on which to compute, e.g. gpu, cpu.
             logging_level: Minimum severity of messages to log. One of the strings
                 INFO, WARNING, DEBUG, ERROR and CRITICAL.
             summary_writer: A `SummaryWriter` to control, among others, log
@@ -76,7 +75,7 @@ class SNRE_B(RatioEstimator):
         num_simulations_per_round: OneOrMore[int],
         x_o: Optional[Tensor] = None,
         num_atoms: int = 10,
-        batch_size: int = 50,
+        training_batch_size: int = 50,
         learning_rate: float = 5e-4,
         validation_fraction: float = 0.1,
         stop_after_epochs: int = 20,
@@ -103,7 +102,7 @@ class SNRE_B(RatioEstimator):
                 sampling so that the simulator is run with parameters that are likely
                 for that `x_o`, i.e. they are sampled from the posterior obtained in the
                 previous round $p(\theta|x_o)$.
-            batch_size: Training batch size.
+            training_batch_size: Training batch size.
             learning_rate: Learning rate for Adam optimizer.
             validation_fraction: The fraction of data to use for validation.
             stop_after_epochs: The number of epochs to wait for improvement on the
