@@ -4,12 +4,14 @@ from sbi.utils.get_nn_models import posterior_nn
 from sbi.utils.io import get_data_root, get_log_root, get_project_root
 from sbi.utils.plot import pairplot
 from sbi.utils.sbiutils import (
-    Standardize,
+    standardizing_transform,
+    standardizing_net,
     sample_posterior_within_prior,
     del_entries,
     clamp_and_warn,
     handle_invalid_x,
     warn_on_invalid_x,
+    x_shape_from_simulation,
 )
 from sbi.utils.torchutils import (
     BoxUniform,
