@@ -4,30 +4,20 @@
 from __future__ import annotations
 
 import pytest
-import torch
-from torch import zeros, ones, eye
+from torch import eye, ones, zeros
 from torch.distributions import MultivariateNormal
 
 import sbi.utils as utils
 from sbi.inference import SNL, prepare_for_sbi
 from sbi.simulators.linear_gaussian import (
-    true_posterior_linear_gaussian_mvn_prior,
-    samples_true_posterior_linear_gaussian_uniform_prior,
-    samples_true_posterior_linear_gaussian_mvn_prior_different_dims,
     diagonal_linear_gaussian,
     linear_gaussian,
+    samples_true_posterior_linear_gaussian_mvn_prior_different_dims,
+    samples_true_posterior_linear_gaussian_uniform_prior,
+    true_posterior_linear_gaussian_mvn_prior,
 )
-from tests.test_utils import get_prob_outside_uniform_prior, check_c2st
-
-# Use cpu by default.
-torch.set_default_tensor_type("torch.FloatTensor")
-
-"""
-Shared seeding in the module
------------------------------
-Some tests in this module have `set_seed` as an argument. This argument points to
-tests/conftest.py to seed the test with the seed set in conftext.py.
-"""
+from tests.test_utils import check_c2st, get_prob_outside_uniform_prior
+from sbi.utils.torchutils import configure_default_device
 
 
 @pytest.mark.parametrize("num_dim", (1, 3))
@@ -69,6 +59,8 @@ def test_c2st_snl_on_linearGaussian_different_dims(set_seed):
         set_seed: fixture for manual seeding
     """
 
+    device = "cpu"
+    configure_default_device(device)
     theta_dim = 3
     x_dim = 2
     discard_dims = theta_dim - x_dim
@@ -102,9 +94,10 @@ def test_c2st_snl_on_linearGaussian_different_dims(set_seed):
         simulation_batch_size=50,
         mcmc_method="slice_np",
         show_progress_bars=False,
+        device=device,
     )
 
-    posterior = infer(num_rounds=1, num_simulations_per_round=4000)  # type: ignore
+    posterior = infer(num_rounds=1, num_simulations_per_round=5000)  # type: ignore
     samples = posterior.sample((num_samples,), x=x_o, thin=3)
 
     # Compute the c2st and assert it is near chance level of 0.5.
