@@ -4,31 +4,24 @@
 from __future__ import annotations
 
 import pytest
-import torch
-from torch import eye, zeros, ones
+from torch import eye, ones, zeros
 from torch.distributions import MultivariateNormal
 
 import sbi.utils as utils
-from tests.test_utils import (
-    check_c2st,
-    get_prob_outside_uniform_prior,
-    get_dkl_gaussian_prior,
-)
-from sbi.inference import SRE, AALR, prepare_for_sbi
-
+from sbi.inference import AALR, SRE, prepare_for_sbi
 from sbi.simulators.linear_gaussian import (
-    true_posterior_linear_gaussian_mvn_prior,
-    samples_true_posterior_linear_gaussian_uniform_prior,
-    samples_true_posterior_linear_gaussian_mvn_prior_different_dims,
     diagonal_linear_gaussian,
     linear_gaussian,
+    samples_true_posterior_linear_gaussian_mvn_prior_different_dims,
+    samples_true_posterior_linear_gaussian_uniform_prior,
+    true_posterior_linear_gaussian_mvn_prior,
 )
-
-# use cpu by default
-torch.set_default_tensor_type("torch.FloatTensor")
-# Seeding:
-# Some tests in this module have "set_seed" as an argument. This argument points to
-# tests/conftest.py to seed the test with the seed set in conftext.py.
+from tests.test_utils import (
+    check_c2st,
+    get_dkl_gaussian_prior,
+    get_prob_outside_uniform_prior,
+)
+from sbi.utils.torchutils import configure_default_device
 
 
 @pytest.mark.parametrize("num_dim", (1, 3))
@@ -67,6 +60,8 @@ def test_c2st_sre_on_linearGaussian_different_dims(set_seed):
         set_seed: fixture for manual seeding
     """
 
+    device = "cpu"
+    configure_default_device(device)
     theta_dim = 3
     x_dim = 2
     discard_dims = theta_dim - x_dim
@@ -102,9 +97,10 @@ def test_c2st_sre_on_linearGaussian_different_dims(set_seed):
         classifier="resnet",
         simulation_batch_size=50,
         show_progress_bars=False,
+        device=device,
     )
 
-    posterior = infer(num_rounds=1, num_simulations_per_round=1000)
+    posterior = infer(num_rounds=1, num_simulations_per_round=5000)
     samples = posterior.sample((num_samples,), x=x_o, thin=3)
 
     # Compute the c2st and assert it is near chance level of 0.5.
@@ -156,11 +152,10 @@ def test_c2st_sre_on_linearGaussian(
     def simulator(theta):
         return linear_gaussian(theta, likelihood_shift, likelihood_cov)
 
-    simulator, prior, x_shape = prepare_for_sbi(simulator, prior)
+    simulator, prior = prepare_for_sbi(simulator, prior)
     kwargs = dict(
         simulator=simulator,
         prior=prior,
-        x_shape=x_shape,
         classifier="resnet",
         simulation_batch_size=50,
         mcmc_method="slice_np",
